/*The MIT License (MIT)

Copyright (c) 2020 Marvel App

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.*/

const MochaJSDelegate = require("./MochaJSDelegate");
const Async = require("sketch/async");
const Settings = require("sketch/settings");
const SketchDOM = require("sketch/dom");

function listen(context) {
  COScript.currentCOScript().setShouldKeepAround_(true);

  let delegate = new MochaJSDelegate({
    "didReceivePluginUserInfo:": function(notification) {
      const userInfo = notification.userInfo();

      // Note this is a NSDictionary not a Javascript Dictionary
      // Therefore don't expect all JS methods related to this object
      const pluginUserInfo = userInfo.plugin_user_info;

      // We just use those as a backup as getDocuments might
      // return copies of files with the same UUID
      const filePath = encodeURI(userInfo.file_path);

      var documents = SketchDOM.getDocuments();

      // Find the document based on a combo of the UUID and file path.
      // We use == because they are different types.
      const document = documents.find(
        doc => doc.id == userInfo.document_uuid && doc.path == filePath
      );

      if (!document) {
        console.log(
          "Document isn't found because it was probably not open anymore..."
        );
        return;
      }

      for (var key in pluginUserInfo) {
        value = pluginUserInfo.objectForKey(key);
        Settings.setDocumentSettingForKey(document, key, value);
      }
    }
  });

  Async.createFiber();

  let delegateInstance = delegate.getClassInstance();

  NSDistributedNotificationCenter.defaultCenter().addObserver_selector_name_object(
    delegateInstance,
    NSSelectorFromString("didReceivePluginUserInfo:"),
    "com.marvelprototyping.sketchmeta",
    nil
  );
}

exports.listen = listen;
